"""Tests for Roku Client Helpers."""
from ipaddress import ip_address
from socket import gaierror
from unittest.mock import AsyncMock

import pytest

from rokuecp.exceptions import RokuConnectionError
from rokuecp.helpers import (
    determine_device_name,
    guess_stream_format,
    is_ip_address,
    resolve_hostname,
)

from . import fake_addrinfo_results

HOSTNAME = "roku.local"
HOST = "192.168.1.2"


def test_determine_device_name() -> None:
    """Test the determine_device_name helper."""
    brand = "Brand"
    assert determine_device_name("", None, None, None) == "Roku (Unknown Name)"
    assert determine_device_name(brand, None, None, None) == "Brand"
    assert determine_device_name(brand, "Friendly Name", None, None) == "Friendly Name"
    assert determine_device_name(brand, None, "Default Name", None) == "Default Name"
    assert determine_device_name(brand, None, None, "Model Name") == "Brand Model Name"


def test_guess_stream_format() -> None:
    """Test the guess_stream_format helper."""
    assert guess_stream_format("/path/media.media") is None
    assert guess_stream_format("/path/media.txt") is None

    assert guess_stream_format("/path/media.mp4", "video/mp4") == "mp4"
    assert guess_stream_format("/path/media.mp4") == "mp4"
    assert guess_stream_format("/path/media.m4v") == "mp4"
    assert guess_stream_format("/path/media.mov") == "mp4"
    assert guess_stream_format("/path/media.mkv") == "mkv"
    assert guess_stream_format("/path/media.mks") == "mks"
    assert guess_stream_format("/path/media.m3u8") == "hls"
    assert guess_stream_format("/path/media.dash") == "dash"
    assert guess_stream_format("/path/media.mpd") == "dash"
    assert guess_stream_format("/path/media.ism/manifest") == "ism"

    assert guess_stream_format("/path/media.mp3") == "mp3"
    assert guess_stream_format("/path/media.m4a") == "m4a"
    assert guess_stream_format("/path/media.mka") == "mka"
    assert guess_stream_format("/path/media.wma") == "wma"


def test_is_ip_address() -> None:
    """Test the is_ip_address helper."""
    assert is_ip_address(HOST)
    assert not is_ip_address(HOSTNAME)


@pytest.mark.asyncio
async def test_resolve_hostname(resolver: AsyncMock) -> None:
    """Test the resolve_hostname helper."""
    resolver.return_value = fake_addrinfo_results([HOST])

    result = await resolve_hostname(HOSTNAME)
    assert result == HOST
    ip_address(result)


@pytest.mark.asyncio
async def test_resolve_hostname_error_invalid(resolver: AsyncMock) -> None:
    """Test the resolve_hostname helper fails properly."""
    resolver.return_value = fake_addrinfo_results(["not-an-ip"])

    with pytest.raises(RokuConnectionError):
        await resolve_hostname("error.local")


@pytest.mark.asyncio
async def test_resolve_hostname_error_not_found(resolver: AsyncMock) -> None:
    """Test the resolve_hostname helper fails properly."""
    resolver.side_effect = gaierror

    with pytest.raises(RokuConnectionError):
        await resolve_hostname("error.local")



"""Tests for Roku."""
from __future__ import annotations

import os
import socket


def fake_addrinfo_results(
    hosts: list[str],
    family: int = socket.AF_INET,
) -> list[tuple[int, None, None, None, list[str | int]]]:
    """Resolve hostname for mocked testing."""
    return [(family, None, None, None, [h, 0]) for h in hosts]


def load_fixture(filename: str) -> str:
    """Load a fixture."""
    path = os.path.join(  # noqa: PTH118
        os.path.dirname(__file__),  # noqa: PTH120
        "fixtures",
        filename,
    )
    with open(path, encoding="utf-8") as fptr:  # noqa: PTH123
        return fptr.read()



"""Setup pytest."""
from collections.abc import Generator
from unittest.mock import AsyncMock, Mock, patch

import pytest


@pytest.fixture(name="resolver")
def resolver_fixture() -> Generator[AsyncMock, None, None]:
    """Mock the threaded resolver."""
    resolver = AsyncMock(return_value=[])
    loop = Mock()
    loop.getaddrinfo = resolver

    with patch("rokuecp.resolver.get_running_loop", return_value=loop):
        yield resolver



# pylint: disable=W0621
"""Asynchronous Python client for Roku."""
import asyncio

from rokuecp import Roku


async def main() -> None:
    """Show example of connecting to your Roku."""
    async with Roku("192.168.1.61") as roku:
        # Get Roku Device Info
        device = await roku.update()
        print(device.info)

        await roku.remote("poweron")
        await asyncio.sleep(2)
        await roku.remote("home")

        # Open Netflix
        await asyncio.sleep(2)
        await roku.launch("12")


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



"""Tests for Roku."""
# pylint: disable=protected-access
from datetime import datetime, timedelta
from unittest.mock import AsyncMock

import pytest
from aiohttp import ClientSession
from aresponses import ResponsesMockServer
from freezegun.api import FrozenDateTimeFactory

from rokuecp import Roku, RokuError, models

from . import fake_addrinfo_results, load_fixture

HOST = "192.168.1.86"
PORT = 8060

MATCH_HOST = f"{HOST}:{PORT}"
ICON_BASE = f"http://{MATCH_HOST}/query/icon"


@pytest.mark.asyncio
async def test_loop() -> None:
    """Test loop usage is handled correctly."""
    async with Roku(HOST) as roku:
        assert isinstance(roku, Roku)


@pytest.mark.asyncio
async def test_app_icon_url() -> None:
    """Test app_icon_url is handled correctly."""
    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        assert roku.app_icon_url("101") == f"{ICON_BASE}/101"


@pytest.mark.asyncio
@pytest.mark.freeze_time("2022-03-27")
async def test_get_dns_state(
    aresponses: ResponsesMockServer,
    resolver: AsyncMock,
    freezer: FrozenDateTimeFactory,
) -> None:
    """Test get_dns_state is handled correctly."""
    aresponses.add(
        f"192.168.1.99:{PORT}",
        "/support/hostname",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    aresponses.add(
        f"192.168.1.89:{PORT}",
        "/support/hostname",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        assert roku.get_dns_state() == {
            "enabled": False,
            "hostname": None,
            "ip_address": None,
            "resolved_at": None,
        }

        roku2 = Roku("roku.dev", session=session)
        assert roku2.get_dns_state() == {
            "enabled": True,
            "hostname": "roku.dev",
            "ip_address": None,
            "resolved_at": None,
        }

        resolver.return_value = fake_addrinfo_results(["192.168.1.99"])
        assert await roku2._request("support/hostname")
        dns = roku2.get_dns_state()
        assert dns["enabled"]
        assert dns["hostname"] == "roku.dev"
        assert dns["ip_address"] == "192.168.1.99"
        assert dns["resolved_at"] == datetime(2022, 3, 27, 0, 0)  # noqa: DTZ001

        resolver.return_value = fake_addrinfo_results(["192.168.1.89"])
        freezer.tick(delta=timedelta(hours=3))
        assert await roku2._request("support/hostname")
        dns = roku2.get_dns_state()
        assert dns["enabled"]
        assert dns["hostname"] == "roku.dev"
        assert dns["ip_address"] == "192.168.1.89"
        assert dns["resolved_at"] == datetime(2022, 3, 27, 3, 0)  # noqa: DTZ001

    aresponses.assert_plan_strictly_followed()

@pytest.mark.asyncio
async def test_device(aresponses: ResponsesMockServer) -> None:
    """Test app property is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/query/device-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("device-info.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("apps.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/active-app",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("active-app-roku.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        await client.update()

        assert client.device
        assert isinstance(client.device, models.Device)

    aresponses.assert_no_unused_routes()
    aresponses.assert_all_requests_matched()


@pytest.mark.asyncio
async def test_launch(aresponses: ResponsesMockServer) -> None:
    """Test launch is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/launch/101",
        "POST",
        aresponses.Response(status=200),
    )

    aresponses.add(
        MATCH_HOST,
        "/launch/102?contentID=deeplink",
        "POST",
        aresponses.Response(status=200),
        match_querystring=True,
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.launch("101")
        await roku.launch("102", {"contentID": "deeplink"})

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_play_on_roku(aresponses: ResponsesMockServer) -> None:
    """Test play_on_roku is handled correctly."""
    video_url = "http://example.com/video file÷awe.mp4?v=2"
    encoded = "http%3A%2F%2Fexample.com%2Fvideo+file%C3%B7awe.mp4%3Fv%3D2"

    aresponses.add(
        MATCH_HOST,
        f"/input/15985?t=v&u={encoded}",
        "POST",
        aresponses.Response(status=200),
        match_querystring=True,
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.play_on_roku(video_url)

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_literal(aresponses: ResponsesMockServer) -> None:
    """Test literal is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_t",
        "POST",
        aresponses.Response(status=200),
    )

    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_h",
        "POST",
        aresponses.Response(status=200),
    )

    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_e",
        "POST",
        aresponses.Response(status=200),
    )

    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_+",
        "POST",
        aresponses.Response(status=200),
    )

    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_@",
        "POST",
        aresponses.Response(status=200),
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.literal("the @")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_remote(aresponses: ResponsesMockServer) -> None:
    """Test remote is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/keypress/Home",
        "POST",
        aresponses.Response(status=200),
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.remote("home")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_remote_invalid_key() -> None:
    """Test remote with invalid key is handled correctly."""
    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            await roku.remote("super")


@pytest.mark.asyncio
async def test_remote_search(aresponses: ResponsesMockServer) -> None:
    """Test remote search keypress is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/search/browse",
        "POST",
        aresponses.Response(status=200),
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.remote("search")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_remote_literal(aresponses: ResponsesMockServer) -> None:
    """Test remote literal keypress is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_t",
        "POST",
        aresponses.Response(status=200),
    )

    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_h",
        "POST",
        aresponses.Response(status=200),
    )

    aresponses.add(
        MATCH_HOST,
        "/keypress/Lit_e",
        "POST",
        aresponses.Response(status=200),
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.remote("Lit_the")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_search(aresponses: ResponsesMockServer) -> None:
    """Test search is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/search/browse?keyword=test",
        "POST",
        aresponses.Response(status=200, text="OK"),
        match_querystring=True,
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.search("test")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_tune(aresponses: ResponsesMockServer) -> None:
    """Test tune is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/launch/tvinput.dtv?ch=13.4",
        "POST",
        aresponses.Response(status=200, text="OK"),
        match_querystring=True,
    )

    async with ClientSession() as session:
        roku = Roku(HOST, session=session)
        await roku.tune("13.4")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_update(aresponses: ResponsesMockServer) -> None:
    """Test update method is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/query/device-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("device-info.xml"),
        ),
        repeat=2,
    )

    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("apps.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/active-app",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("active-app-roku.xml"),
        ),
        repeat=2,
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client.update()

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert isinstance(response.app, models.Application)
        assert response.channel is None
        assert response.media is None

        assert response.state.available
        assert not response.state.standby
        assert len(response.channels) == 0

        response = await client.update()

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert isinstance(response.app, models.Application)
        assert response.channel is None
        assert response.media is None

        assert response.state.available
        assert not response.state.standby
        assert len(response.channels) == 0

    aresponses.assert_no_unused_routes()
    aresponses.assert_all_requests_matched()


@pytest.mark.asyncio
async def test_update_media_state(aresponses: ResponsesMockServer) -> None:
    """Test update method is handled correctly with pluto app."""
    aresponses.add(
        MATCH_HOST,
        "/query/device-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("device-info.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("apps.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/active-app",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("active-app-pluto.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/media-player",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("media-player-pluto-play.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client.update()

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.media, models.MediaState)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert isinstance(response.app, models.Application)
        assert response.channel is None

        assert response.state.available
        assert not response.state.standby
        assert len(response.channels) == 0

        assert not response.media.live
        assert not response.media.paused
        assert response.media.duration == 6496
        assert response.media.position == 38

    aresponses.assert_no_unused_routes()
    aresponses.assert_all_requests_matched()


@pytest.mark.asyncio
async def test_update_power_off(aresponses: ResponsesMockServer) -> None:
    """Test update method is handled correctly when power is off."""
    aresponses.add(
        MATCH_HOST,
        "/query/device-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("device-info-power-off.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("apps.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client.update()

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert response.app is None
        assert response.channel is None
        assert response.media is None

        assert response.state.available
        assert response.state.standby
        assert len(response.channels) == 0

    aresponses.assert_no_unused_routes()
    aresponses.assert_all_requests_matched()


@pytest.mark.asyncio
async def test_update_standby(aresponses: ResponsesMockServer) -> None:
    """Test update method is handled correctly when device transitions to standby."""
    aresponses.add(
        MATCH_HOST,
        "/query/device-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("device-info.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("apps.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/active-app",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("active-app-pluto.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/media-player",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("media-player-pluto-play.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/device-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("device-info-standby.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client.update()

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.media, models.MediaState)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert isinstance(response.app, models.Application)
        assert response.channel is None

        assert response.state.available
        assert not response.state.standby
        assert len(response.channels) == 0

        assert not response.media.live
        assert not response.media.paused
        assert response.media.duration == 6496
        assert response.media.position == 38

        response = await client.update()

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert response.app is None
        assert response.channel is None
        assert response.media is None

        assert response.state.available
        assert response.state.standby
        assert len(response.channels) == 0

    aresponses.assert_no_unused_routes()
    aresponses.assert_all_requests_matched()


@pytest.mark.asyncio
async def test_update_tv(aresponses: ResponsesMockServer) -> None:
    """Test update method is handled correctly for TVs."""
    for _ in range(2):
        aresponses.add(
            MATCH_HOST,
            "/query/device-info",
            "GET",
            aresponses.Response(
                status=200,
                headers={"Content-Type": "application/xml"},
                text=load_fixture("device-info-7820x.xml"),
            ),
        )

        aresponses.add(
            MATCH_HOST,
            "/query/apps",
            "GET",
            aresponses.Response(
                status=200,
                headers={"Content-Type": "application/xml"},
                text=load_fixture("apps-tv.xml"),
            ),
        )

        aresponses.add(
            MATCH_HOST,
            "/query/active-app",
            "GET",
            aresponses.Response(
                status=200,
                headers={"Content-Type": "application/xml"},
                text=load_fixture("active-app-tv.xml"),
            ),
        )

        aresponses.add(
            MATCH_HOST,
            "/query/tv-active-channel",
            "GET",
            aresponses.Response(
                status=200,
                headers={"Content-Type": "application/xml"},
                text=load_fixture("tv-active-channel.xml"),
            ),
        )

    aresponses.add(
        MATCH_HOST,
        "/query/tv-channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("tv-channels.xml"),
        ),
    )

    aresponses.add(
        MATCH_HOST,
        "/query/tv-channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("tv-channels-single.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client.update()

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert isinstance(response.app, models.Application)
        assert isinstance(response.channel, models.Channel)
        assert response.media is None

        assert response.state.available
        assert not response.state.standby
        assert len(response.channels) == 2

        response = await client.update(True)  # noqa: FBT003

        assert response
        assert isinstance(response.info, models.Info)
        assert isinstance(response.state, models.State)
        assert isinstance(response.apps, list)
        assert isinstance(response.channels, list)
        assert isinstance(response.app, models.Application)
        assert isinstance(response.channel, models.Channel)
        assert response.media is None

        assert response.state.available
        assert not response.state.standby
        assert len(response.channels) == 1

    aresponses.assert_no_unused_routes()
    aresponses.assert_all_requests_matched()


@pytest.mark.asyncio
async def test_get_active_app(aresponses: ResponsesMockServer) -> None:
    """Test _get_active_app method is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/query/active-app",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("active-app-amazon.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        assert await client._get_active_app()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_active_app_invalid(aresponses: ResponsesMockServer) -> None:
    """Test _get_active_app method is handled correctly with invalid data."""
    aresponses.add(
        MATCH_HOST,
        "/query/active-app",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<other>value</other>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._get_active_app()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_apps(aresponses: ResponsesMockServer) -> None:
    """Test _get_apps method is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("apps.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        res = await client._get_apps()
        assert isinstance(res, list)
        assert len(res) == 8

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_apps_invalid(aresponses: ResponsesMockServer) -> None:
    """Test _get_apps method is handled correctly with invalid data."""
    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<other>value</other>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._get_apps()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_apps_single_app(aresponses: ResponsesMockServer) -> None:
    """Test _get_apps method is handled correctly with single app."""
    aresponses.add(
        MATCH_HOST,
        "/query/apps",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("apps-single.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        res = await client._get_apps()
        assert isinstance(res, list)
        assert len(res) == 1

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_device_info(aresponses: ResponsesMockServer) -> None:
    """Test _get_device_info method is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/query/device-info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<other>value</other>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._get_device_info()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_media_state_close(aresponses: ResponsesMockServer) -> None:
    """Test _get_media_state method is handled correctly with closed media."""
    aresponses.add(
        MATCH_HOST,
        "/query/media-player",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("media-player-close.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        assert await client._get_media_state()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_media_state_invalid(aresponses: ResponsesMockServer) -> None:
    """Test _get_media_state method is handled correctly with invalid data."""
    aresponses.add(
        MATCH_HOST,
        "/query/media-player",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<other>value</other>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._get_media_state()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_media_state_live(aresponses: ResponsesMockServer) -> None:
    """Test _get_media_state method is handled correctly with live media."""
    aresponses.add(
        MATCH_HOST,
        "/query/media-player",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("media-player-pluto-live.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        assert await client._get_media_state()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_media_state_pause(aresponses: ResponsesMockServer) -> None:
    """Test _get_media_state method is handled correctly with paused media."""
    aresponses.add(
        MATCH_HOST,
        "/query/media-player",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("media-player-pluto-pause.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        assert await client._get_media_state()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_media_state_play(aresponses: ResponsesMockServer) -> None:
    """Test _get_media_state method is handled correctly with playing media."""
    aresponses.add(
        MATCH_HOST,
        "/query/media-player",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("media-player-pluto-play.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        assert await client._get_media_state()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_tv_active_channel(aresponses: ResponsesMockServer) -> None:
    """Test _get_tv_active_channel method is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/query/tv-active-channel",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<other>value</other>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._get_tv_active_channel()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_tv_channels(aresponses: ResponsesMockServer) -> None:
    """Test _get_tv_channels method is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/query/tv-channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<other>value</other>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._get_tv_channels()

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_tv_channels_no_channels(aresponses: ResponsesMockServer) -> None:
    """Test _get_tv_channels method is handled correctly with no channels."""
    aresponses.add(
        MATCH_HOST,
        "/query/tv-channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("tv-channels-empty.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        res = await client._get_tv_channels()
        assert isinstance(res, list)
        assert len(res) == 0

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_get_tv_channels_single_channel(aresponses: ResponsesMockServer) -> None:
    """Test _get_tv_channels method is handled correctly with single channel."""
    aresponses.add(
        MATCH_HOST,
        "/query/tv-channels",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text=load_fixture("tv-channels-single.xml"),
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        res = await client._get_tv_channels()
        assert isinstance(res, list)
        assert len(res) == 1

    aresponses.assert_plan_strictly_followed()



"""Tests for Roku."""
# pylint: disable=protected-access
import asyncio
from datetime import datetime, timedelta
from socket import gaierror
from unittest.mock import AsyncMock, patch

import pytest
from aiohttp import ClientError, ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer
from freezegun.api import FrozenDateTimeFactory

from rokuecp import Roku
from rokuecp.exceptions import (
    RokuConnectionError,
    RokuConnectionTimeoutError,
    RokuError,
)
from tests import fake_addrinfo_results

HOSTNAME = "roku.local"
HOST = "192.168.1.86"
PORT = 8060

MATCH_HOST = f"{HOST}:{PORT}"
NON_STANDARD_PORT = 3333


@pytest.mark.asyncio
async def test_xml_request(aresponses: ResponsesMockServer) -> None:
    """Test XML response is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/response/xml",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<status>OK</status>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client._request("response/xml")

        assert isinstance(response, dict)
        assert response["status"] == "OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_text_xml_request(aresponses: ResponsesMockServer) -> None:
    """Test (text) XML response is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/response/text-xml",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "text/xml"},
            text="<status>OK</status>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client._request("response/text-xml")

        assert isinstance(response, dict)
        assert response["status"] == "OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_xml_request_parse_error(aresponses: ResponsesMockServer) -> None:
    """Test invalid XML response is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/response/xml-parse-error",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<!status>>",
        ),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._request("response/xml-parse-error")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_text_request(aresponses: ResponsesMockServer) -> None:
    """Test non XML response is handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/response/text",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client._request("response/text")
        assert response == "OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_internal_session(aresponses: ResponsesMockServer) -> None:
    """Test JSON response is handled correctly with internal session."""
    aresponses.add(
        MATCH_HOST,
        "/response/xml",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/xml"},
            text="<status>OK</status>",
        ),
    )

    async with Roku(HOST) as client:
        response = await client._request("response/xml")

        assert isinstance(response, dict)
        assert response["status"] == "OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_post_request(aresponses: ResponsesMockServer) -> None:
    """Test POST requests are handled correctly."""
    aresponses.add(
        MATCH_HOST,
        "/method/post",
        "POST",
        aresponses.Response(status=200, text="OK"),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        response = await client._request("method/post", method="POST")
        assert response == "OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_request_port(aresponses: ResponsesMockServer) -> None:
    """Test the handling of non-standard API port."""
    aresponses.add(
        f"{HOST}:{NON_STANDARD_PORT}",
        "/support/port",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    async with ClientSession() as session:
        client = Roku(host=HOST, port=NON_STANDARD_PORT, session=session)
        response = await client._request("support/port")
        assert response == "OK"

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_timeout(aresponses: ResponsesMockServer) -> None:
    """Test request timeout from the API."""

    # Faking a timeout by sleeping
    async def response_handler(_: ClientResponse) -> Response:
        await asyncio.sleep(2)
        return aresponses.Response(body="Timeout!")

    aresponses.add(
        MATCH_HOST,
        "/timeout",
        "GET",
        response_handler,
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session, request_timeout=1)
        with pytest.raises(RokuConnectionTimeoutError):
            assert await client._request("timeout")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_client_error() -> None:
    """Test HTTP client error."""
    async with ClientSession() as session:
        with patch.object(session, "request") as mock:
            mock.side_effect = ClientError

            client = Roku(HOST, session=session)
            with pytest.raises(RokuConnectionError):
                assert await client._request("client/error", method="ABC")


@pytest.mark.asyncio
async def test_http_error404(aresponses: ResponsesMockServer) -> None:
    """Test HTTP 404 response handling."""
    aresponses.add(
        MATCH_HOST,
        "/http/404",
        "GET",
        aresponses.Response(text="Not Found!", status=404),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._request("http/404")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_http_error500(aresponses: ResponsesMockServer) -> None:
    """Test HTTP 500 response handling."""
    aresponses.add(
        MATCH_HOST,
        "/http/500",
        "GET",
        aresponses.Response(text="Internal Server Error", status=500),
    )

    async with ClientSession() as session:
        client = Roku(HOST, session=session)
        with pytest.raises(RokuError):
            assert await client._request("http/500")

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
@pytest.mark.freeze_time("2022-03-27 00:00:00+00:00")
async def test_resolve_hostname(
    aresponses: ResponsesMockServer,
    resolver: AsyncMock,
    freezer: FrozenDateTimeFactory,
) -> None:
    """Test that hostnames are resolved before request."""
    resolver.return_value = fake_addrinfo_results([HOST])

    aresponses.add(
        MATCH_HOST,
        "/support/hostname",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    aresponses.add(
        f"192.168.1.68:{PORT}",
        "/support/hostname",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    async with ClientSession() as session:
        client = Roku(HOSTNAME, session=session)
        assert await client._request("support/hostname")

        dns = client.get_dns_state()
        assert dns["enabled"]
        assert dns["hostname"] == HOSTNAME
        assert dns["ip_address"] == HOST
        assert dns["resolved_at"] == datetime(2022, 3, 27, 0, 0)  # noqa: DTZ001

        freezer.tick(delta=timedelta(hours=3))
        resolver.return_value = fake_addrinfo_results(["192.168.1.68"])
        assert await client._request("support/hostname")

        dns = client.get_dns_state()
        assert dns["enabled"]
        assert dns["hostname"] == HOSTNAME
        assert dns["ip_address"] == "192.168.1.68"
        assert dns["resolved_at"] == datetime(2022, 3, 27, 3, 0)  # noqa: DTZ001

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
@pytest.mark.freeze_time("2022-03-27 00:00:00+00:00")
async def test_resolve_hostname_multiple_clients(
    aresponses: ResponsesMockServer,
    resolver: AsyncMock,
) -> None:
    """Test that hostnames are resolved before request with multiple clients."""
    aresponses.add(
        MATCH_HOST,
        "/support/hostname",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    aresponses.add(
        f"192.168.1.99:{PORT}",
        "/support/hostname",
        "GET",
        aresponses.Response(status=200, text="OK"),
    )

    async with ClientSession() as session:
        resolver.return_value = fake_addrinfo_results([HOST])
        client = Roku(HOSTNAME, session=session)
        assert await client._request("support/hostname")

        dns = client.get_dns_state()
        assert dns["enabled"]
        assert dns["hostname"] == HOSTNAME
        assert dns["ip_address"] == HOST
        assert dns["resolved_at"] == datetime(2022, 3, 27, 0, 0)  # noqa: DTZ001

        resolver.return_value = fake_addrinfo_results(["192.168.1.99"])
        client2 = Roku("roku.dev", session=session)
        assert await client2._request("support/hostname")

        dns2 = client2.get_dns_state()
        assert dns2["enabled"]
        assert dns2["hostname"] == "roku.dev"
        assert dns2["ip_address"] == "192.168.1.99"
        assert dns2["resolved_at"] == datetime(2022, 3, 27, 0, 0)  # noqa: DTZ001

    aresponses.assert_plan_strictly_followed()


@pytest.mark.asyncio
async def test_resolve_hostname_error(resolver: AsyncMock) -> None:
    """Test that hostname resolution errors are handled."""
    resolver.side_effect = gaierror

    async with ClientSession() as session:
        client = Roku(HOSTNAME, session=session)

        with pytest.raises(RokuConnectionError):
            await client._request("support/hostname-error")



"""Tests for Roku Models."""
from datetime import datetime

import pytest
import xmltodict

from rokuecp import RokuError, models

from . import load_fixture

ACTIVE_APP_NETFLIX = xmltodict.parse(load_fixture("active-app-netflix.xml"))
ACTIVE_APP_PLUTO = xmltodict.parse(load_fixture("active-app-pluto.xml"))
ACTIVE_APP_TV = xmltodict.parse(load_fixture("active-app-tvinput-dtv.xml"))
APPS = xmltodict.parse(load_fixture("apps.xml"))
APPS_TV = xmltodict.parse(load_fixture("apps-tv.xml"))
DEVICE_INFO = xmltodict.parse(load_fixture("device-info.xml"))
DEVICE_INFO_3500X = xmltodict.parse(load_fixture("device-info-3500x.xml"))
DEVICE_INFO_7820X = xmltodict.parse(load_fixture("device-info-7820x.xml"))
DEVICE_INFO_C147X = xmltodict.parse(load_fixture("device-info-c147x.xml"))
DEVICE_INFO_D803X = xmltodict.parse(load_fixture("device-info-d803x.xml"))
MEDIA_PLAYER_CLOSE = xmltodict.parse(load_fixture("media-player-close.xml"))
MEDIA_PLAYER_PLUTO_LIVE = xmltodict.parse(load_fixture("media-player-pluto-live.xml"))
MEDIA_PLAYER_PLUTO_PAUSE = xmltodict.parse(load_fixture("media-player-pluto-pause.xml"))
MEDIA_PLAYER_PLUTO_PLAY = xmltodict.parse(load_fixture("media-player-pluto-play.xml"))
TV_ACTIVE_CHANNEL = xmltodict.parse(load_fixture("tv-active-channel.xml"))
TV_CHANNELS = xmltodict.parse(load_fixture("tv-channels.xml"))

DEVICE = {
    "info": DEVICE_INFO["device-info"],
    "apps": APPS["apps"]["app"],
    "app": ACTIVE_APP_PLUTO["active-app"],
    "media": MEDIA_PLAYER_PLUTO_PLAY["player"],
    "available": True,
    "standby": False,
}

DEVICE_TV = {
    "info": DEVICE_INFO_7820X["device-info"],
    "apps": APPS_TV["apps"]["app"],
    "app": ACTIVE_APP_TV["active-app"],
    "channels": TV_CHANNELS["tv-channels"]["channel"],
    "channel": TV_ACTIVE_CHANNEL["tv-channel"]["channel"],
    "available": True,
    "standby": False,
}


def test_device() -> None:
    """Test the Device model."""
    device = models.Device(DEVICE)

    assert device

    assert device.info
    assert isinstance(device.info, models.Info)

    assert device.state
    assert isinstance(device.state, models.State)

    assert device.apps
    assert isinstance(device.apps, list)

    assert device.app
    assert isinstance(device.app, models.Application)

    assert device.media
    assert isinstance(device.media, models.MediaState)

    assert isinstance(device.channels, list)
    assert len(device.channels) == 0

    assert device.channel is None


def test_device_no_data() -> None:
    """Test the Device model with no device info."""
    with pytest.raises(RokuError):
        models.Device({})


def test_device_tv() -> None:
    """Test the Device model with Roku TV."""
    device = models.Device(DEVICE_TV)

    assert device

    assert device.info
    assert isinstance(device.info, models.Info)

    assert device.state
    assert isinstance(device.state, models.State)

    assert device.apps
    assert isinstance(device.apps, list)

    assert device.app
    assert isinstance(device.app, models.Application)
    assert device.app.app_id == "tvinput.dtv"

    assert device.media is None

    assert isinstance(device.channels, list)
    assert len(device.channels) == 2

    assert isinstance(device.channel, models.Channel)


def test_device_as_dict() -> None:
    """Test the dictionary version of Device."""
    device = models.Device(DEVICE)
    assert device

    device_dict = device.as_dict()
    assert device_dict
    assert isinstance(device_dict, dict)
    assert isinstance(device_dict["info"], dict)
    assert isinstance(device_dict["state"], dict)
    assert isinstance(device_dict["apps"], list)
    assert len(device_dict["apps"]) == 8
    assert device_dict["app"]
    assert isinstance(device_dict["app"], dict)
    assert device_dict["channel"] is None
    assert device_dict["media"]
    assert isinstance(device_dict["media"], dict)
    assert isinstance(device_dict["channels"], list)
    assert len(device_dict["channels"]) == 0


def test_device_tv_as_dict() -> None:
    """Test the dictionary version of Device."""
    device = models.Device(DEVICE_TV)
    assert device

    device_dict = device.as_dict()
    assert device_dict
    assert isinstance(device_dict, dict)
    assert isinstance(device_dict["info"], dict)
    assert isinstance(device_dict["state"], dict)
    assert isinstance(device_dict["apps"], list)
    assert len(device_dict["apps"]) == 10
    assert device_dict["app"]
    assert isinstance(device_dict["app"], dict)
    assert device_dict["channel"]
    assert isinstance(device_dict["channel"], dict)
    assert device_dict["media"] is None
    assert isinstance(device_dict["channels"], list)
    assert len(device_dict["channels"]) == 2


def test_info() -> None:
    """Test the Info model."""
    info = models.Info.from_dict(DEVICE_INFO["device-info"])

    assert info
    assert info.name == "My Roku 3"
    assert info.brand == "Roku"
    assert info.device_location is None
    assert info.device_type == "box"
    assert info.network_type == "ethernet"
    assert info.network_name is None
    assert info.model_name == "Roku 3"
    assert info.model_number == "4200X"
    assert info.serial_number == "1GU48T017973"
    assert info.ethernet_support
    assert info.ethernet_mac == "b0:a7:37:96:4d:fa"
    assert info.wifi_mac == "b0:a7:37:96:4d:fb"
    assert not info.supports_airplay
    assert not info.supports_find_remote
    assert not info.supports_private_listening
    assert not info.supports_wake_on_wlan
    assert not info.headphones_connected
    assert info.version == "7.5.0"


def test_info_without_client_device_name() -> None:
    """Test the Info model without a client device name."""
    temp = {
        **DEVICE_INFO["device-info"],
        "user-device-name": None,
        "friendly-device-name": None,
        "default-device-name": None,
    }

    info = models.Info.from_dict(temp)

    assert info
    assert info.name == "Roku Roku 3"


def test_info_stick_3500x() -> None:
    """Test the Info model with Roku Stick."""
    info = models.Info.from_dict(DEVICE_INFO_3500X["device-info"])

    assert info
    assert info.name == "My Roku Stick"
    assert info.brand == "Roku"
    assert info.device_location is None
    assert info.device_type == "stick"
    assert info.network_type == "wifi"
    assert info.network_name == "NetworkSSID"
    assert info.model_name == "Roku Stick"
    assert info.model_number == "3500X"
    assert info.serial_number == "2L647N055555"
    assert not info.ethernet_support
    assert info.ethernet_mac is None
    assert info.wifi_mac == "b0:a7:37:6a:ec:2d"
    assert not info.supports_airplay
    assert info.supports_find_remote
    assert not info.supports_private_listening
    assert not info.supports_wake_on_wlan
    assert not info.headphones_connected
    assert info.version == "10.0.0"


def test_info_tv_7820x() -> None:
    """Test the Info model with TV model 7820X."""
    info = models.Info.from_dict(DEVICE_INFO_7820X["device-info"])

    assert info
    assert info.name == '58" Onn Roku TV'
    assert info.brand == "Onn"
    assert info.device_location == "Living room"
    assert info.device_type == "tv"
    assert info.network_type == "wifi"
    assert info.network_name == "NetworkSSID"
    assert info.model_name == "100005844"
    assert info.model_number == "7820X"
    assert info.serial_number == "YN00H5555555"
    assert info.ethernet_support
    assert info.ethernet_mac == "d4:3a:2e:07:fd:cb"
    assert info.wifi_mac == "d8:13:99:f8:b0:c6"
    assert info.supports_airplay
    assert info.supports_find_remote
    assert info.supports_private_listening
    assert info.supports_wake_on_wlan
    assert not info.headphones_connected
    assert info.version == "9.2.0"


def test_info_tv_c147x() -> None:
    """Test the Info model with TV model C147X."""
    info = models.Info.from_dict(DEVICE_INFO_C147X["device-info"])

    assert info

    assert info.name == "TCL•Roku TV - X000008YG08J"
    assert info.brand == "TCL"
    assert info.device_location is None
    assert info.device_type == "tv"
    assert info.network_type == "wifi"
    assert info.network_name == "NetworkSSID"
    assert info.model_name == "55S20"
    assert info.model_number == "C147X"
    assert info.serial_number == "X000008YG08J"
    assert info.ethernet_support
    assert info.ethernet_mac == "34:51:80:6d:01:42"
    assert info.wifi_mac == "d4:ab:cd:f8:e1:d5"
    assert info.supports_airplay
    assert not info.supports_find_remote
    assert info.supports_private_listening
    assert info.supports_wake_on_wlan
    assert not info.headphones_connected
    assert info.version == "11.5.0"


def test_info_tv_d803x() -> None:
    """Test the Info model with TV model D803X."""
    info = models.Info.from_dict(DEVICE_INFO_D803X["device-info"])

    assert info
    assert info.name == '42" Onn Roku TV'
    assert info.brand == "onn."
    assert info.device_location is None
    assert info.device_type == "tv"
    assert info.network_type == "wifi"
    assert info.network_name == "NetworkSSID"
    assert info.model_name == "100018254"
    assert info.model_number == "D803X"
    assert info.serial_number == "X00755550DCH"
    assert not info.ethernet_support
    assert info.ethernet_mac is None
    assert info.wifi_mac == "d4:ab:cd:2f:6b:55"
    assert info.supports_airplay
    assert not info.supports_find_remote
    assert info.supports_private_listening
    assert info.supports_wake_on_wlan
    assert not info.headphones_connected
    assert info.version == "10.0.0"


def test_application() -> None:
    """Test the Application model."""
    app = models.Application.from_dict(APPS["apps"]["app"][0])

    assert app
    assert app.app_id == "11"
    assert app.name == "Roku Channel Store"


def test_application_active_app() -> None:
    """Test the Application model with active app."""
    app = models.Application.from_dict(ACTIVE_APP_NETFLIX["active-app"])

    assert app
    assert app.app_id == "12"
    assert app.name == "Netflix"
    assert app.version == "4.1.218"


def test_channel() -> None:
    """Test the Channel model."""
    channel = models.Channel.from_dict(TV_CHANNELS["tv-channels"]["channel"][0])

    assert channel
    assert channel.name == "WhatsOn"
    assert channel.number == "1.1"
    assert channel.channel_type == "air-digital"
    assert not channel.hidden
    assert channel.program_title is None
    assert channel.program_description is None
    assert channel.program_rating is None
    assert channel.signal_mode is None
    assert channel.signal_strength is None


def test_channel_active_tv() -> None:
    """Test the Channel model with active TV channel."""
    channel = models.Channel.from_dict(TV_ACTIVE_CHANNEL["tv-channel"]["channel"])
    description = """The team will travel all around the world in order to shut down
a global crime ring."""

    assert channel
    assert channel.name == "getTV"
    assert channel.number == "14.3"
    assert channel.channel_type == "air-digital"
    assert not channel.hidden
    assert channel.program_title == "Airwolf"
    assert channel.program_description == description.replace("\n", " ")
    assert channel.program_rating == "TV-14-D-V"
    assert channel.signal_mode == "480i"
    assert channel.signal_strength == -75


def test_channel_active_tv_signal_stength_non_numeric() -> None:
    """Test the Channel model with active TV channel and non-numeric signal strength."""
    active_channel = {
        **TV_ACTIVE_CHANNEL["tv-channel"]["channel"],
        "signal-strength": "n/a",
    }

    channel = models.Channel.from_dict(active_channel)
    description = """The team will travel all around the world in order to shut down
a global crime ring."""

    assert channel
    assert channel.name == "getTV"
    assert channel.number == "14.3"
    assert channel.channel_type == "air-digital"
    assert not channel.hidden
    assert channel.program_title == "Airwolf"
    assert channel.program_description == description.replace("\n", " ")
    assert channel.program_rating == "TV-14-D-V"
    assert channel.signal_mode == "480i"
    assert channel.signal_strength is None


def test_media_state_close() -> None:
    """Test the MediaState model with closed media."""
    state = models.MediaState.from_dict(MEDIA_PLAYER_CLOSE["player"])

    assert state is None


def test_media_state_live() -> None:
    """Test the MediaState model with live media."""
    state = models.MediaState.from_dict(MEDIA_PLAYER_PLUTO_LIVE["player"])

    assert state
    assert isinstance(state.at, datetime)

    assert not state.paused
    assert state.live
    assert state.duration == 95
    assert state.position == 73


def test_media_state_pause() -> None:
    """Test the MediaState model with paused media."""
    state = models.MediaState.from_dict(MEDIA_PLAYER_PLUTO_PAUSE["player"])

    assert state
    assert isinstance(state.at, datetime)

    assert state.paused
    assert not state.live
    assert state.duration == 6496
    assert state.position == 313


def test_media_state_play() -> None:
    """Test the MediaState model with playing media."""
    state = models.MediaState.from_dict(MEDIA_PLAYER_PLUTO_PLAY["player"])

    assert state
    assert isinstance(state.at, datetime)

    assert not state.paused
    assert not state.live
    assert state.duration == 6496
    assert state.position == 38


def test_state() -> None:
    """Test the State model."""
    state = models.State(available=True, standby=False)

    assert state
    assert isinstance(state.at, datetime)
